--Create Users
USE AdventureWorks
GO

CREATE USER BOSS WITHOUT LOGIN;
CREATE USER Emp1 WITHOUT LOGIN;
CREATE USER Emp2 WITHOUT LOGIN;

--Create base table
CREATE TABLE Sales
(
    OrderID INT,
    SalesPerson sysname,
    Product VARCHAR(10),
    Qty INT
);

--Insert some data
INSERT Sales
VALUES
(1, 'Emp1', 'Bike', 15),
(2, 'Emp1', 'Car', 3),
(3, 'Emp1', 'Boat', 14),
(4, 'Emp2', 'Car', 4),
(5, 'Emp2', 'Boat', 7),
(6, 'Emp2', 'Bike', 9);

-- View the 6 rows in the table  
SELECT *
FROM Sales;

--Grant access
GRANT SELECT ON Sales TO BOSS;
GRANT SELECT ON Sales TO Emp1;
GRANT SELECT ON Sales TO Emp2;

--Create new schema and inline value functions
CREATE SCHEMA Security;  
GO  

CREATE FUNCTION Security.fn_securitypredicate(@SalesPerson AS sysname)  
    RETURNS TABLE  
WITH SCHEMABINDING  
AS  
    RETURN SELECT 1 AS fn_securitypredicate_result   
WHERE @SalesPerson = USER_NAME() OR USER_NAME() = 'Boss';

--Create security policy adding the function as a filter predicate, state must be ON to enable
CREATE SECURITY POLICY SalesPersonFilter
ADD FILTER PREDICATE SECURITY.fn_securitypredicate (SalesPerson)
ON dbo.Sales
WITH
(STATE = ON);

--View data as individual users
EXECUTE AS USER = 'Emp1';
SELECT *
FROM Sales;
REVERT;

EXECUTE AS USER = 'Emp2';
SELECT *
FROM Sales;
REVERT;

EXECUTE AS USER = 'Boss';
SELECT *
FROM Sales;
REVERT;

--Disable the policy and view again
ALTER SECURITY POLICY SalesPersonFilter WITH (STATE = OFF);


--Clean Up

DROP SECURITY POLICY SalesPersonFilter
DROP FUNCTION SECURITY.fn_securitypredicate
DROP SCHEMA SECURITY
DROP TABLE SALES
DROP USER BOSS
DROP USER Emp1
DROP USER Emp2

